@@ -16,6 +16,15 @@ class IntegrationTestScrubbers < Loofah::TestCase
   NOFOLLOW_FRAGMENT = '<a href="http://www.example.com/">Click here</a>'
   NOFOLLOW_RESULT = '<a href="http://www.example.com/" rel="nofollow">Click here</a>'
 
+  TARGET_FRAGMENT = '<a href="http://www.example.com/">Click here</a>'
+  TARGET_RESULT = '<a href="http://www.example.com/" target="_blank">Click here</a>'
+
+  ANCHOR_TARGET_FRAGMENT = '<a href="#top">Click here</a>'
+  ANCHOR_TARGET_RESULT = '<a href="#top">Click here</a>'
+
+  TARGET_WITH_TOP_FRAGMENT = '<a href="http://www.example.com/" target="_top">Click here</a>'
+  TARGET_WITH_TOP_RESULT = '<a href="http://www.example.com/" target="_blank">Click here</a>'
+
   NOFOLLOW_WITH_REL_FRAGMENT = '<a href="http://www.example.com/" rel="noopener">Click here</a>'
   NOFOLLOW_WITH_REL_RESULT = '<a href="http://www.example.com/" rel="noopener nofollow">Click here</a>'
 
@@ -25,6 +34,12 @@ class IntegrationTestScrubbers < Loofah::TestCase
   NOOPENER_WITH_REL_FRAGMENT = '<a href="http://www.example.com/" rel="nofollow">Click here</a>'
   NOOPENER_WITH_REL_RESULT = '<a href="http://www.example.com/" rel="nofollow noopener">Click here</a>'
 
+  NOREFERRER_FRAGMENT = '<a href="http://www.example.com/">Click here</a>'
+  NOREFERRER_RESULT = '<a href="http://www.example.com/" rel="noreferrer">Click here</a>'
+
+  NOREFERRER_WITH_REL_FRAGMENT = '<a href="http://www.example.com/" rel="noopener">Click here</a>'
+  NOREFERRER_WITH_REL_RESULT = '<a href="http://www.example.com/" rel="noopener noreferrer">Click here</a>'
+
   UNPRINTABLE_FRAGMENT = "<b>Lo\u2029ofah ro\u2028cks!</b><script>x\u2028y</script>"
   UNPRINTABLE_RESULT = "<b>Loofah rocks!</b><script>xy</script>"
 
@@ -35,6 +50,9 @@ class IntegrationTestScrubbers < Loofah::TestCase
   ENTITY_HACK_ATTACK_TEXT_SCRUB = "Hack attack!&lt;script&gt;alert('evil')&lt;/script&gt;"
   ENTITY_HACK_ATTACK_TEXT_SCRUB_UNESC = "Hack attack!<script>alert('evil')</script>"
 
+  BREAKPOINT_FRAGMENT = "<p>Some text here in a logical paragraph.<br><br>Some more text, apparently a second paragraph.<br><br>Et cetera...</p>"
+  BREAKPOINT_RESULT = "<p>Some text here in a logical paragraph.</p><p>Some more text, apparently a second paragraph.</p><p>Et cetera...</p>"
+
   context "scrubbing shortcuts" do
     context "#scrub_document" do
       it "is a shortcut for parse-and-scrub" do
@@ -182,6 +200,36 @@ class IntegrationTestScrubbers < Loofah::TestCase
           end
         end
 
+        context ":targetblank" do
+          context "when target is not set" do
+            it "adds a target='_blank' attribute to hyperlinks" do
+              doc = klass.parse("<html><body>#{TARGET_FRAGMENT}</body></html>")
+              result = doc.scrub!(:targetblank)
+
+              assert_equal TARGET_RESULT, doc.xpath("/html/body").inner_html
+              assert_equal doc, result
+            end
+
+            it "skips target attribute when linking to anchor" do
+              doc = klass.parse("<html><body>#{ANCHOR_TARGET_FRAGMENT}</body></html>")
+              result = doc.scrub!(:targetblank)
+
+              assert_equal ANCHOR_TARGET_RESULT, doc.xpath("/html/body").inner_html
+              assert_equal doc, result
+            end
+          end
+
+          context "when target is set" do
+            it "replaces existing 'target' attribute with '_blank' to hyperlinks" do
+              doc = klass.parse("<html><body>#{TARGET_WITH_TOP_FRAGMENT}</body></html>")
+              result = doc.scrub!(:targetblank)
+
+              assert_equal TARGET_WITH_TOP_RESULT, doc.xpath("/html/body").inner_html
+              assert_equal doc, result
+            end
+          end
+        end
+
         context ":unprintable" do
           it "removes unprintable unicode characters" do
             doc = klass.parse("<html><body>#{UNPRINTABLE_FRAGMENT}</body></html>")
@@ -191,6 +239,16 @@ class IntegrationTestScrubbers < Loofah::TestCase
             assert_equal doc, result
           end
         end
+
+        context ":double_breakpoint" do
+          it "replaces double line breaks with paragraph tags" do
+            doc = klass.parse("<html><body>#{BREAKPOINT_FRAGMENT}</body></html>")
+            result = doc.scrub!(:double_breakpoint)
+
+            assert_equal BREAKPOINT_RESULT, doc.xpath("/html/body").inner_html.delete("\n")
+            assert_equal doc, result
+          end
+        end
       end
 
       context "#text" do
@@ -415,6 +473,28 @@ class IntegrationTestScrubbers < Loofah::TestCase
           end
         end
 
+        context ":noreferrer" do
+          context "for a hyperlink without a 'rel' attribute" do
+            it "add a 'noreferrer' attribute to hyperlinks" do
+              doc = klass.parse("<div>#{NOREFERRER_FRAGMENT}</div>")
+              result = doc.scrub!(:noreferrer)
+
+              assert_equal NOREFERRER_RESULT, doc.xpath("./div").inner_html
+              assert_equal doc, result
+            end
+          end
+
+          context "for a hyperlink that does have a rel attribute" do
+            it "appends 'noreferrer' to 'rel' attribute" do
+              doc = klass.parse("<div>#{NOREFERRER_WITH_REL_FRAGMENT}</div>")
+              result = doc.scrub!(:noreferrer)
+
+              assert_equal NOREFERRER_WITH_REL_RESULT, doc.xpath("./div").inner_html
+              assert_equal doc, result
+            end
+          end
+        end
+
         context ":unprintable" do
           it "removes unprintable unicode characters" do
             doc = klass.parse("<div>#{UNPRINTABLE_FRAGMENT}</div>")
